#define F_CPU F_OSC
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include "uart.h"
#include "spi.h"

#define sbi(reg, bit) reg |= (1<<bit)
#define cbi(reg, bit) reg &= ~(1<<bit)

//#define GET_UBRR_FROM_BAUDRATE(BAUD) ((F_OSC + BAUD*8)/(16*BAUD) - 1)
// for atmega 16
#define GET_UBRR_FROM_BAUDRATE(BAUD) F_OSC/16/BAUD-1

void delay(void)
{
	_delay_ms(2000);
}

void binary_to_uart(int val)
{
	for(unsigned char i = 0; i < 16; ++i)
	{
		if(((val << i) & 0x8000) == 0)
			uart_transmit('0');
		else
			uart_transmit('1');
	}
	
	uart_transmit('\n');
}

void rfm12_init(void)
{
	int rx = 0;
	
	sbi(DDRB, PB3);								// FSK
	cbi(PORTB, PB3);
	
	uart_transmit('i');
	uart_transmit('n');
	uart_transmit('i');
	uart_transmit('t');
	uart_transmit('\n');
	
	delay();
	
	rx = spi_wtransfer(0x80D7);		// enable FIFO
	binary_to_uart(rx);
	rx = spi_wtransfer(0xC0E0);		// CLK: 10 MHz
	binary_to_uart(rx);
	rx = spi_wtransfer(0xC2AB);		// data filter: internal
	binary_to_uart(rx);
	rx = spi_wtransfer(0xCA81);		// set FIFO mode
	binary_to_uart(rx);
	rx = spi_wtransfer(0xE000);		// disable wake-up-timer
	binary_to_uart(rx);
	rx = spi_wtransfer(0xC800);		// disable low-duty-cycle
	binary_to_uart(rx);
	rx = spi_wtransfer(0xC4F7);		// afc settings: autotuning: -10kHz...+7,5kHz
	binary_to_uart(rx);
	rx = spi_wtransfer(0xA640);		// freq = 334 MHz
	binary_to_uart(rx);
	rx = spi_wtransfer(0x94A0);		// vdi, fast, 134kHz BW, 0dBm, -103dB
	binary_to_uart(rx);
	rx = spi_wtransfer(0xC647);		// 4.8 kb/s baudrate
	binary_to_uart(rx);
	rx = spi_wtransfer(0x9850);		// MAX OUT
	binary_to_uart(rx);
	
	uart_transmit('\n');
	
	rx = spi_wtransfer(0x0000);
	binary_to_uart(rx);
	uart_transmit('\n');

}

void rfm12_wait(void)
{
	cbi(PO_SPI, P_MOSI);
	cbi(PO_SPI, P_SS);
	while((PI_SPI & (1 << P_MISO)));
	
}

void rfm12_send(void)
{
	int rx = 0;
	
	rx = spi_wtransfer(0x8238);		// activate tx
	binary_to_uart(rx);
	rfm12_wait();
	rx = spi_wtransfer(0xB8AA);
	binary_to_uart(rx);
	rfm12_wait();
	rx = spi_wtransfer(0xB8AA);
	binary_to_uart(rx);
	rfm12_wait();
	rx = spi_wtransfer(0xB8AA);
	binary_to_uart(rx);
	rfm12_wait();
	rx = spi_wtransfer(0xB82D);
	binary_to_uart(rx);
	rfm12_wait();
	rx = spi_wtransfer(0xB8D4);
	binary_to_uart(rx);
	rfm12_wait();
	rx = spi_wtransfer(0xB801);
	binary_to_uart(rx);
	rfm12_wait();
	rx = spi_wtransfer(0xB802);
	binary_to_uart(rx);
	rfm12_wait();
	rx = spi_wtransfer(0xB803);
	binary_to_uart(rx);
	rfm12_wait();
	rx = spi_wtransfer(0xB804);
	binary_to_uart(rx);
	rfm12_wait();
	rx = spi_wtransfer(0x8208);
	binary_to_uart(rx);
	rfm12_wait();
}

int main (void)
{
	int number = 0;
  DDRD  = (1 << PD5) | (1 << PD6);
  
  cli();
  
  delay();
  sbi(PORTD, PD5);
  sbi(PORTD, PD6);
  delay();
  cbi(PORTD, PD5);
  cbi(PORTD, PD6);
  delay();
  
  spi_init_master();
  uart_init(GET_UBRR_FROM_BAUDRATE(9600));
  
  sbi(PORTD, PD5);
  delay();
  
  rfm12_init();
  
  sbi(PORTD, PD6);
  cbi(PORTD, PD5);
  delay();
  
  while(1)
  {
  	sbi(PORTD, PD5);
  	delay();
  	rfm12_send();
  	cbi(PORTD, PD5);
  	delay();
  	
  }
  
  return 0;
}
