#include "spi.h"
#include <avr/io.h>

#define sbi(reg, bit) reg |= (1<<bit)
#define cbi(reg, bit) reg &= ~(1<<bit)

void spi_init_master(void)
{
	unsigned char tmp = 0;
	
  // MOSI, SCK and SlaveSelect are output MISO is input others remain unchanged
  DDR_SPI |= ((1 << DD_MOSI) | (1 << DD_SCK) | (1 << DD_SS));
  cbi(DDR_SPI, DD_MISO);
  
  // 
  sbi(PO_SPI, P_SS);
  
  // enable SPI, master, clock rate F_OSC/16, 
  // CPOL = 0 CPHA = 0
  SPCR = (1 << SPE) | (1 << MSTR) | (1 << SPR0);
  
  tmp = SPSR;
  tmp = SPDR;
}

unsigned char spi_btransfer(unsigned char data)
{
	cbi(PO_SPI, P_SS);
  //Start new transmission
  SPDR = data;
  
  // wait until finished
  while(!(SPSR & (1 << SPIF)));
  
  sbi(PO_SPI, P_SS);
  
  return SPDR;
}

unsigned int spi_wtransfer(const unsigned int data)
{
	unsigned char data_lo = (unsigned char)data;
	unsigned char data_hi = (unsigned char)(data >> 8);
	
	cbi(PO_SPI, P_SS);
	
	SPDR = data_hi;
	
	while(!(SPSR & (1 << SPIF)));
	
	data_hi = SPDR;
	
	SPDR = data_lo;
	
	while(!(SPSR & (1 << SPIF)));
	
	data_lo = SPDR;
	
	sbi(PO_SPI, P_SS);
	
	return (data_hi << 8) | (data_lo);
}

