#include "uart.h"
#include <avr/io.h>

void uart_init(unsigned long speed_setting)
{
  // set baudrate (UBRR Register)
  UBRRH = (unsigned char)((speed_setting >> 8) & 0x7F);				
  UBRRL = (unsigned char)speed_setting;
  
  // Enable reiceive & transmitt
  UCSRB = (1<<RXEN)|(1<<TXEN);
  
  // Use 8 databits + 2 stop bits
  // URSEL     =   1 -> UCSRC should be written to
  // UMSEL     =   0 -> Asynchronouse Mode
  // UPM[1:0]  =  00 -> no parity
  // USBS      =   1 -> 2 stop bits
  // UCSZ[2:0] = 011 -> 8 bit
  // UCPOL     =   0 -> transmit on rising edge / receive on falling edge
  
  UCSRC = (1<<URSEL)|(1<<USBS)|(3<<UCSZ0);

}

void uart_transmit(char data_byte)
{
	// wait until last transmit completed (blocking)
	while(!(UCSRA & (1 << UDRE)));
	
	UDR = data_byte;
}

unsigned char uart_peek(void)
{
	if((UCSRA & (1 << RXC)))
		return 1;
	else
		return 0;
}

void uart_wait_tx(void)
{
	while(!(UCSRA & (1 << TXC)));
}

unsigned char uart_receive(void)
{
	// wait until receive is complete (blocking)
	while(!(UCSRA & (1 << RXC)));
	
	return UDR;
}

